unit BinaryFile;
{*****************************************************************************}
{* Komponente	TBinaryFile																			25.05.1998 ame *}
{*****************************************************************************}
{* Dient als POT fr beliebige Dateien.                                      *}
{*****************************************************************************}
{* by ADRIAN MEYER 20.07.1998 Version 1.0                                    *}
{* adrian.meyer@berner.ch																		                 *}
{* Please report heavy bugs.                                                 *}
{*****************************************************************************}
interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
	Dialogs, dsgnintf, StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
	TBinaryFile = class;
	TBinaryFileEditOperation = (eoNone, eoLoad, eoSave, eoDelete );

	TfrmFileInfoEditor = class(TForm)
		dlgOpen: TOpenDialog;
    Bevel1: TBevel;
    pnlToolbar: TPanel;
		btnLaden: TSpeedButton;
		btnSpeichern: TSpeedButton;
		btnLoeschen: TSpeedButton;
		Bevel2: TBevel;
		Panel1: TPanel;
		Panel2: TPanel;
		memFileInfo: TMemo;
		dlgSave: TSaveDialog;
		StatusBar1: TStatusBar;
		Panel3: TPanel;
		btnOK: TBitBtn;
		btnAbbrechen: TBitBtn;
		Image1: TImage;
		procedure btnLadenClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure btnLoeschenClick(Sender: TObject);
		procedure btnSpeichernClick(Sender: TObject);
	private
		{ Private-Deklarationen }
		fComp: TBinaryFile;
		fOperation: TBinaryFileEditOperation;
		procedure SetButtons;
		procedure SetList;

	public
		{ Public-Deklarationen }
		function ShowModal: TModalResult;
		property Comp: TBinaryFile read fComp;
	end;

	TBinaryFile = class(TComponent)
	private
		{ Private-Deklarationen }
		fData: TMemoryStream;
		fFileDate: TDateTime;
		fFileName: string;
		fFilePath: string;
		fFileSize: longint;

		procedure ReadData(Stream: TStream);
		procedure WriteData(Stream: TStream);

	protected
		{ Protected-Deklarationen }
		procedure DefineProperties( Filer: TFiler ); override;

	public
		{ Public-Deklarationen }
		constructor Create( aOwner: TComponent ); override;
		destructor Destroy; override;

		procedure LoadFromFile( FileName: string );
		procedure SaveToFile( FileName: string );
		procedure LoadFromStream( Stream: TStream );
		procedure SaveToStream( Stream: TStream );
		function SaveToTempFile: string;

		procedure Clear;
		function Empty: boolean;

		procedure Assign( Value: TPersistent ); override;
	published
		property FileDate: TDateTime read fFileDate write fFileDate;
		property FileName: string read fFileName write fFileName;
		property FilePath: string read fFilePath write fFilePath;
		property FileSize: longint read fFileSize write fFileSize;

		{ Published-Deklarationen }
	end;

	TBinaryFileItemEditor = class( TStringProperty )
	public
		function GetAttributes: TPropertyAttributes; override;
		function GetValue: string; override;
		procedure Edit; override;
	end;

procedure Register;

implementation

{$R *.DFM}

{*****************************************************************************}
{* Konstruktor																								25.05.1998 ame *}
{*****************************************************************************}
constructor TBinaryFile.Create( aOwner: TComponent );
begin
	fData := TMemoryStream.Create;
	inherited Create( aOwner );													{ Vorfahr aufrufen }
end;

{*****************************************************************************}
{* Destruktor																									25.05.1998 ame *}
{*****************************************************************************}
destructor TBinaryFile.Destroy;
begin
	fData.Destroy;
	inherited Destroy;
end;

{*****************************************************************************}
{* Daten Definieren																						15.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.DefineProperties( Filer: TFiler );
begin
	Filer.DefineBinaryProperty( 'Data', ReadData, WriteData, not Empty );
end;

{*****************************************************************************}
{* Daten lesen																								15.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.ReadData(Stream: TStream);
begin
	fData.LoadFromStream( Stream );
end;

{*****************************************************************************}
{* Daten schreiben																						15.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.WriteData(Stream: TStream);
begin
	fData.SaveToStream( Stream );
end;

{*****************************************************************************}
{* Datei leer?																								11.06.1998 ame *}
{*****************************************************************************}
function TBinaryFile.Empty: boolean;
begin
	Result := fData.Size = 0;
end;

{*****************************************************************************}
{* Datei lschen																							11.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.Clear;
begin
	fData.Clear;
	fFileDate := 0;
	fFileName := '';
	fFIlePath := '';
	fFileSize := 0;
end;

{*****************************************************************************}
{* .1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.Assign( Value: TPersistent );
var
	ts: TMemoryStream;
begin
	ts := TMemoryStream.Create;
	fFileName := TBinaryFile( Value ).FileName;
	fFileSize := TBinaryFile( Value ).FileSize;
	fFilePath := TBinaryFile( Value ).FilePath;
	fFileDate := TBinaryFile( Value ).FileDate;

	TBinaryFile( Value ).SaveToStream( ts );
	fData.LoadFromStream( ts );
	ts.Destroy;
end;

{*****************************************************************************}
{* File laden																									11.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.LoadFromFile( FileName: string );
var
	SearchRec: TSearchRec;
begin
	if FileExists( FileName ) then
	begin
		FindFirst( FileName, faAnyFile, SearchRec );
		fData.LoadFromFile( FileName );

		fFileDate := FileDateToDateTime( SearchRec.Time );
		fFileName := ExtractFileName( FileName );
		fFIlePath := copy( ExtractFilePath( FileName ), 1, length( ExtractFilePath( FileName )) - 1 );
		fFileSize := SearchRec.Size;
	end;
end;

{*****************************************************************************}
{* File Speichern																							11.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.SaveToFile( FileName: string );
begin
	if not Empty then	fData.SaveToFile( FileName );
end;

{*****************************************************************************}
{* In ein Tempfile speichern																	15.06.1998 ame *}
{*****************************************************************************}
function TBinaryFile.SaveToTempFile: string;
var
	aFileName: array[ 0..512 ] of char;
begin
	GetTempPath( 512, @aFileName );
	GetTempFileName( @aFileName , '~bf', 0, @aFileName );
	fData.SaveToFile( StrPas( aFileName ));
	Result := StrPas( aFileName );
end;

{*****************************************************************************}
{* Stream laden																								15.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.LoadFromStream( Stream: TStream );
begin
	fData.LoadFromStream( Stream );
end;

{*****************************************************************************}
{* Stream Speichern																						15.06.1998 ame *}
{*****************************************************************************}
procedure TBinaryFile.SaveToStream( Stream: TStream );
begin
	TMemoryStream( Stream ).LoadFromStream( fData );
end;

{*****************************************************************************}
{* Attributte ermitteln																				11.06.1998 ame *}
{*****************************************************************************}
function TBinaryFileItemEditor.GetAttributes: TPropertyAttributes;
begin
	Result := [ paDialog ];
end;

{*****************************************************************************}
{* Anzeige Objektinspektor																		11.06.1998 ame *}
{*****************************************************************************}
function TBinaryFileItemEditor.GetValue: string;
begin
	if TBinaryFile( GetComponent( 0 )).Empty then
		Result := '(Leer)'
	else
		Result := TBinaryFile( GetComponent( 0 )).FileName;
end;

{*****************************************************************************}
{* Formular fr FileInfo Editor																25.05.1998 ame *}
{*****************************************************************************}
procedure TBinaryFileItemEditor.Edit;
begin
	with TBinaryFile( GetComponent( 0 )), TfrmFileInfoEditor.Create( nil ) do
	begin
		Comp.Assign( TBinaryFile( GetComponent( 0 )) );

		if ShowModal = mrOK then
		begin
			TBinaryFile( GetComponent( 0 )).Assign( Comp );
			Modified;
		end;

		Destroy;
	end;
end;

{*****************************************************************************}
{* .1998 ame *}
{*****************************************************************************}
procedure Register;
begin
	RegisterComponents( 'ADVICE', [ TBinaryFile ]);
	RegisterPropertyEditor( TypeInfo( string ), TBinaryFile, 'FileName', TBinaryFileItemEditor );
end;

{*****************************************************************************}
{* Editor erstellen																						12.06.1998 ame *}
{*****************************************************************************}
procedure TfrmFileInfoEditor.FormCreate(Sender: TObject);
begin
	fComp := TBinaryFile.Create( self );
end;

{*****************************************************************************}
{* Editor lschen																							12.06.1998 ame *}
{*****************************************************************************}
procedure TfrmFileInfoEditor.FormDestroy(Sender: TObject);
begin
	fComp.Destroy;
end;

{*****************************************************************************}
{* Editor anzeigen																						12.06.1998 ame *}
{*****************************************************************************}
function TfrmFileInfoEditor.ShowModal: TModalResult;
begin
	SetList;
{	if TempForm.ShowModal = mrOK then
		Comp.LoadFromFile( TempForm.dlgOpen.FileName );}

	fOperation := eoNone;
	SetButtons;
	Result := inherited ShowModal;
end;

{*****************************************************************************}
{* File ffnen																								25.05.1998 ame *}
{*****************************************************************************}
procedure TfrmFileInfoEditor.btnLadenClick(Sender: TObject);
begin
	with dlgOpen do																			{ Memo und Dialog }
	if Execute then
	begin
		Comp.LoadFromFile( FileName );
		fOperation := eoLoad;
		SetList;
		SetButtons;
	end;
end;

{*****************************************************************************}
{* Schalter setzen																						12.06.1998 ame *}
{*****************************************************************************}
procedure TfrmFileInfoEditor.SetButtons;
begin
	btnSpeichern.Enabled := not Comp.Empty;
	btnLoeschen.Enabled := not Comp.Empty;
	btnOK.Enabled := fOperation <> eoNone;
end;

procedure TfrmFileInfoEditor.btnLoeschenClick(Sender: TObject);
begin
	Comp.Clear;
	fOperation := eoDelete;
	SetList;
	SetButtons;
end;

procedure TfrmFileInfoEditor.SetList;
begin
	with memFileInfo.Lines do
	begin
		Clear;
		if Comp.FIleDate <> 0 then
			Add( format( '%s'+#9+'='+#9+'%s', [ 'FileDate', DateTimeToStr( Comp.FileDate ) ]))
		else
			Add( 'FileDate'+#9+'=' );
		Add( format( '%s'+#9+'='+#9+'%s', [ 'FileName', Comp.FileName ]));
		Add( format( '%s'+#9+'='+#9+'%s', [ 'FilePath', Comp.FilePath ]));
		Add( format( '%s'+#9+'='+#9+'%d', [ 'FileSize', Comp.FileSize ]));
	end;
end;


procedure TfrmFileInfoEditor.btnSpeichernClick(Sender: TObject);
begin
	if dlgSave.Execute then
		Comp.SaveToFile( dlgSave.FileName );
	fOperation := eoSave;
end;

end.
